package com.oqpo.api.service.stinfo;


import com.oqpo.api.client.oqlink.OqlinkClient;
import com.oqpo.api.client.oqlink.dto.OqScmMenuModel;
import com.oqpo.api.client.oqlink.dto.OqScmMenuRegReqModel;
import com.oqpo.api.client.oqlink.dto.OqScmMenuRegResModel;
import com.oqpo.api.client.oqlink.util.HmacUtil;
import com.oqpo.api.constant.ApiConstants;
import com.oqpo.api.entity.oper.FileEntity;
import com.oqpo.api.entity.stinfo.*;
import com.oqpo.api.entity.stockmng.WhsMgntBaseInfoEntity;
import com.oqpo.api.enums.*;
import com.oqpo.api.exception.GlobalException;
import com.oqpo.api.mapper.FileMapper;
import com.oqpo.api.mapper.stinfo.BrandMapper;
import com.oqpo.api.mapper.stinfo.ItemMapper;
import com.oqpo.api.mapper.stinfo.ItemUnitMapper;
import com.oqpo.api.mapper.stockmng.WhsMngMapper;
import com.oqpo.api.service.CommonService;
import com.oqpo.api.service.SessionService;
import com.oqpo.api.util.StringUtil;
import com.oqpo.api.web.dto.request.GridRequest;
import com.oqpo.api.web.dto.request.stinfo.item.*;
import com.oqpo.api.web.dto.response.GridResponse;
import com.oqpo.api.web.dto.response.stinfo.item.*;
import com.oqpo.api.web.dto.response.stinfo.itemUnit.StroeWhsListResponse;
import com.oqpo.api.web.dto.response.stinfo.itemUnit.StroeWishListResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Slf4j
public class ItemService extends CommonService {

    @Autowired
    private ItemMapper itemMapper;

    @Autowired
    private BrandMapper brandMapper;

    @Autowired
    private ItemUnitMapper itemUnitMapper;

    @Autowired
    private WhsMngMapper whsMngMapper;

    @Autowired
    private FileMapper fileMapper;

    @Autowired
    private SessionService sessionService;

    @Autowired
    private OqlinkClient oqlinkClient;

    /*
      팝업 품목 리스트 조회
     */
    public GridResponse selectPopItemGridList(String sBrandId, String sStoreId, String sPodrPssblDvsn, String sItemNm,
                                              String sSpplyId, String sWhsId, String sLocation, GridRequest gridRequest) throws Exception {
        int gridPage = gridRequest.getGridPage();
        int gridSize = gridRequest.getGridSize();

        int gridRecords = 0;
        int gridTotal = 0;
        List<ItemUnitEntity> entities = new ArrayList<ItemUnitEntity>();

        if (StringUtil.isEmpty(sStoreId)) {
            gridRecords = itemUnitMapper.selectPopItemBrandUnitGridCnt(sBrandId, sItemNm, sPodrPssblDvsn, sSpplyId, sWhsId, sLocation);
            gridTotal = fnCalculateGridTotal(gridSize, gridRecords);
            entities = itemUnitMapper.selectPopItemBrandUnitGridList(sBrandId, sItemNm, sPodrPssblDvsn, sSpplyId, sWhsId, sLocation, gridRequest);
        } else {
            gridRecords = itemUnitMapper.selectPopItemStoreUnitGridCnt(sBrandId, sStoreId, sItemNm, sPodrPssblDvsn);
            gridTotal = fnCalculateGridTotal(gridSize, gridRecords);
            entities = itemUnitMapper.selectPopItemStoreUnitGridList(sBrandId, sStoreId, sItemNm, sPodrPssblDvsn, gridRequest);
        }

        List<Object> gridRows = entities.stream()
                .map(m -> ItemPopSearchResponse.builder()
                        .viewCd("R")
                        .itemId(m.getItemId())
                        .itemNm(m.getItemNm())
                        .podrPssblDvsn(m.getPodrPssblDvsn())
                        .podrPssblDvsnNm(m.getPodrPssblDvsnNm())
                        .unit(m.getUnit())
                        .unitNm(m.getUnitNm())
                        .unitAmt(m.getUnitAmt())
                        .stddQty(m.getStddQty())
                        .unitGubun(m.getUnitGubun())
                        .spplyId(m.getSpplyId())
                        .spplyNm(m.getSpplyNm())
                        .brandUnitUnqNo(m.getBrandUnitUnqNo())
                        .storeUnitUnqNo(m.getStoreUnitUnqNo() == null ? "" : m.getStoreUnitUnqNo())
                        .whsPassQty(m.getWhsPassQty() == null ? 0 : m.getWhsPassQty())
                        .build())
                .collect(Collectors.toList());
        return GridResponse.toDTO(gridPage, gridTotal, gridRecords, gridRows);
    }

    /*
      팝업 신규 품목 리스트 조회
     */
    public GridResponse selectPopNewItemGridList(String sBrandId, String sStoreId, String sPodrPssblDvsn, String sItemNm, String sSpplyId, String sWhsId, String sLocation, GridRequest gridRequest) throws Exception {
        int gridPage = gridRequest.getGridPage();
        int gridSize = gridRequest.getGridSize();

        int gridRecords = 0;
        int gridTotal = 0;
        List<ItemUnitEntity> entities = new ArrayList<ItemUnitEntity>();

        if (StringUtil.isEmpty(sStoreId)) {
            gridRecords = itemUnitMapper.selectPopNewItemBrandUnitGridCnt(sBrandId, sItemNm, sPodrPssblDvsn, sSpplyId, sWhsId, sLocation);
            gridTotal = fnCalculateGridTotal(gridSize, gridRecords);
            entities = itemUnitMapper.selectPopNewItemBrandUnitGridList(sBrandId, sItemNm, sPodrPssblDvsn, sSpplyId, sWhsId, sLocation, gridRequest);
        } else {
            gridRecords = itemUnitMapper.selectPopNewItemStoreUnitGridCnt(sBrandId, sStoreId, sItemNm, sPodrPssblDvsn);
            gridTotal = fnCalculateGridTotal(gridSize, gridRecords);
            entities = itemUnitMapper.selectPopNewItemStoreUnitGridList(sBrandId, sStoreId, sItemNm, sPodrPssblDvsn, gridRequest);
        }

        List<Object> gridRows = entities.stream()
                .map(m -> ItemPopSearchResponse.builder()
                        .viewCd("R")
                        .itemId(m.getItemId())
                        .itemNm(m.getItemNm())
                        .podrPssblDvsn(m.getPodrPssblDvsn())
                        .podrPssblDvsnNm(m.getPodrPssblDvsnNm())
                        .unit(m.getUnit())
                        .unitNm(m.getUnitNm())
                        .unitAmt(m.getUnitAmt())
                        .stddQty(m.getStddQty())
                        .unitGubun(m.getUnitGubun())
                        .spplyId(m.getSpplyId())
                        .brandUnitUnqNo(m.getBrandUnitUnqNo())
                        .storeUnitUnqNo(m.getStoreUnitUnqNo() == null ? "" : m.getStoreUnitUnqNo())
                        .whsPassQty(m.getWhsPassQty() == null ? 0 : m.getWhsPassQty())
                        .build())
                .collect(Collectors.toList());
        return GridResponse.toDTO(gridPage, gridTotal, gridRecords, gridRows);
    }

    /*
      품목 그리드 리스트 조회
     */
    public GridResponse selectItemGridList(String sBrandId, String sItemStCd, String sPodrPssblDvsn, String sItemType,
                                           String sItemGubun, String sClass1, String sClass2, String sClass3, String sClass4, String sItemNm,
                                           String uploadYn, GridRequest gridRequest) throws Exception {
        int gridPage = gridRequest.getGridPage();
        int gridSize = gridRequest.getGridSize();

        int gridRecords = 0;
        int gridTotal = 0;
        List<ItemEntity> entities = new ArrayList<ItemEntity>();

        if (StringUtil.isEmpty(uploadYn)) {
            gridRecords = itemMapper.selectItemGridCnt(sBrandId, "", sItemStCd, sPodrPssblDvsn, sItemType, sItemGubun, sClass1, sClass2, sClass3, sClass4, sItemNm);
            gridTotal = fnCalculateGridTotal(gridSize, gridRecords);
            entities = itemMapper.selectItemGridList(sBrandId, "", sItemStCd, sPodrPssblDvsn, sItemType, sItemGubun, sClass1, sClass2, sClass3, sClass4, sItemNm, gridRequest);
        }

        List<Object> gridRows = entities.stream()
                .map(m -> ItemSearchListResponse.builder()
                        .viewCd("R")
                        .itemId(m.getItemId())
                        .itemNm(m.getItemNm() == null ? "" : m.getItemNm())
                        .itemType(m.getItemType() == null ? "" : m.getItemType())
                        .itemTypeNm(m.getItemTypeNm() == null ? "" : m.getItemTypeNm())
                        .itemClassMgntNo(m.getItemClassInfo() == null ? "" : m.getItemClassInfo().getItemClassMgntNo() == null ? "" : m.getItemClassInfo().getItemClassMgntNo())
                        .itemClassNm(m.getItemClassMngInfo() == null ? "" : m.getItemClassMngInfo().getItemClassNm() == null ? "" : m.getItemClassMngInfo().getItemClassNm())
                        .itemStCd(m.getItemStCd() == null ? "" : m.getItemStCd())
                        .itemStNm(m.getItemStNm() == null ? "" : m.getItemStNm())
                        .podrPssblDvsn(m.getPodrPssblDvsn() == null ? "" : m.getPodrPssblDvsn())
                        .podrPssblDvsnNm(m.getPodrPssblDvsnNm() == null ? "" : m.getPodrPssblDvsnNm())
                        .unit(m.getUnit() == null ? "" : m.getUnit())
                        .unitNm(m.getUnitNm() == null ? "" : m.getUnitNm())
                        .pchItemYn(m.getPchItemYn() == null ? "" : m.getPchItemYn())
                        .saleItemYn(m.getSaleItemYn() == null ? "" : m.getSaleItemYn())
                        .build())
                .collect(Collectors.toList());
        return GridResponse.toDTO(gridPage, gridTotal, gridRecords, gridRows);
    }

    /*
      품목 그리드 리스트 조회
     */
    public GridResponse selectSpplyItemGridList(String sBrandId, String sSpplyId, String sItemStCd, String sPodrPssblDvsn, String sItemNm, GridRequest gridRequest) throws Exception {
        int gridPage = gridRequest.getGridPage();
        int gridSize = gridRequest.getGridSize();

        int gridRecords = itemMapper.selectItemGridCnt(sBrandId, sSpplyId, sItemStCd, sPodrPssblDvsn, "", "", "", "", "", "", sItemNm);
        int gridTotal = fnCalculateGridTotal(gridSize, gridRecords);
        List<ItemEntity> entities = itemMapper.selectItemGridList(sBrandId, sSpplyId, sItemStCd, sPodrPssblDvsn, "", "", "", "", "", "", sItemNm, gridRequest);
        List<Object> gridRows = entities.stream()
                .map(m -> ItemSearchListResponse.builder()
                        .viewCd("R")
                        .itemId(m.getItemId())
                        .itemNm(m.getItemNm() == null ? "" : m.getItemNm())
                        .itemType(m.getItemType() == null ? "" : m.getItemType())
                        .itemTypeNm(m.getItemTypeNm() == null ? "" : m.getItemTypeNm())
                        .itemClassMgntNo(m.getItemClassInfo() == null ? "" : m.getItemClassInfo().getItemClassMgntNo() == null ? "" : m.getItemClassInfo().getItemClassMgntNo())
                        .itemClassNm(m.getItemClassMngInfo() == null ? "" : m.getItemClassMngInfo().getItemClassNm() == null ? "" : m.getItemClassMngInfo().getItemClassNm())
                        .itemStCd(m.getItemStCd() == null ? "" : m.getItemStCd())
                        .itemStNm(m.getItemStNm() == null ? "" : m.getItemStNm())
                        .podrPssblDvsn(m.getPodrPssblDvsn() == null ? "" : m.getPodrPssblDvsn())
                        .podrPssblDvsnNm(m.getPodrPssblDvsnNm() == null ? "" : m.getPodrPssblDvsnNm())
                        .unit(m.getUnit() == null ? "" : m.getUnit())
                        .unitNm(m.getUnitNm() == null ? "" : m.getUnitNm())
                        .pchItemYn(m.getPchItemYn() == null ? "" : m.getPchItemYn())
                        .saleItemYn(m.getSaleItemYn() == null ? "" : m.getSaleItemYn())
                        .build())
                .collect(Collectors.toList());
        return GridResponse.toDTO(gridPage, gridTotal, gridRecords, gridRows);
    }

    /* 품목 정보 */
    public ItemInfoResponse selectItemInfo(String brandId, String itemId) throws Exception {

        ItemEntity entity = itemMapper.selectItemInfo(brandId, itemId);
        List<FileEntity> fileImageInfoList = null;

        if (!StringUtil.isEmpty(entity.getItemImageFileNo())) {
            fileImageInfoList = fileMapper.selectFileList(entity.getItemImageFileNo());
        } else {
            fileImageInfoList = new ArrayList<FileEntity>();
        }

        return ItemInfoResponse.toDTO(entity, fileImageInfoList);

    }

    /* 품목 저장 */
    @Transactional
    public void saveItemInfo(String userId, SaveItemInfolRequest saveItemInfolRequest) throws Exception {
        try {
            ItemEntity entity = toItemEntity(saveItemInfolRequest);
            if ("C".equals(saveItemInfolRequest.getViewCd())) {
                // 품목 아이디 부여  22  entity.setContentsNo(fnGetKeyNo(17));
                //entity.setItemId(fnGetDealNo(26,""));  // 품목아이디 생성
                itemMapper.insertItemInfo(userId, entity);
            } else if ("U".equals(saveItemInfolRequest.getViewCd())) {
                itemMapper.updateItemInfo(userId, entity);
                itemMapper.deleteItemClassInfo(entity.getItemId());
            }

            // 품목 class 정보 등록
            if (!"".toString().equals(saveItemInfolRequest.getItemClass1())) {
                ItemClassEntity iClassEntity = toItemClassEntity(saveItemInfolRequest, entity);
                itemMapper.insertItemClassInfo(userId, iClassEntity);
            }

            // 오더퀸 연동
            if (!StringUtil.isEmpty(entity.getOqBrandId())) {
                List<OqScmMenuModel> menuList = new ArrayList<>();
                OqScmMenuModel menu = new OqScmMenuModel();
                menu.setItemId(entity.getItemId()); // 품목아이디
                menu.setItemNm(entity.getItemNm()); // 품목명
                menu.setBarcodeNo(entity.getBarcode()); // 바코드번호
                menu.setMenuCd(entity.getOqMenuCd()); // 메뉴코드
                // menu.setSupplyPrice(0); // 공급가격 (알수없음)
                // menu.setOrderUnitQty(0); // 발주단위수량 (알수없음)
                menu.setOrderPossibleYn("PO01".equals(entity.getPodrPssblDvsn()) ? "Y" : "N"); // 발주가능여부
                menuList.add(menu);

                OqScmMenuRegReqModel req = new OqScmMenuRegReqModel();
                req.setBrandCd(entity.getOqBrandId());
                req.setMenuList(menuList);

                OqScmMenuRegResModel res = null;
                if ("C".equals(saveItemInfolRequest.getViewCd())) {
                    res = oqlinkClient.regScmMenu(HmacUtil.makeHmacValue4OqLink(req), req);
                } else if ("U".equals(saveItemInfolRequest.getViewCd())) {
                    res = oqlinkClient.changeScmMenu(HmacUtil.makeHmacValue4OqLink(req), req);
                }
                if (SystemMessageCode.BO_IF_OK.getId() == Integer.parseInt(res.getCode())) {
                    // 오더퀸 메뉴코드 업데이트
                    ItemEntity itemEntity = null;
                    for (OqScmMenuModel item : res.getMenuList()) {
                        itemEntity = toItemEntity4OqLink(item);
                        itemMapper.updateItemInfo4OqLinik(userId, itemEntity);
                    }
                } else {
                    throw new GlobalException(SystemMessageCode.valueOf(Integer.valueOf(res.getCode())));
                }
            }
        } catch (GlobalException e) {
            e.getStackTrace();
            throw new GlobalException(e.getSystemMessageCode());
        } catch (Exception e) {
            e.getStackTrace();
            throw new RuntimeException();
        }
    }

    private ItemClassEntity toItemClassEntity(SaveItemInfolRequest saveItemInfolRequest, ItemEntity entity) throws Exception {
        ItemClassEntity iClassEntity = new ItemClassEntity();
        iClassEntity.setItemId(entity.getItemId());
        iClassEntity.setItemNm(entity.getItemNm());
        iClassEntity.setItemClass1(saveItemInfolRequest.getItemClass1());
        iClassEntity.setItemClass2(saveItemInfolRequest.getItemClass2());
        iClassEntity.setItemClass3(saveItemInfolRequest.getItemClass3());
        iClassEntity.setItemClass4(saveItemInfolRequest.getItemClass4());
        ItemClassMngEntity itemClassInfo = itemMapper.selectItemClassMngNo(entity.getBrandId(), iClassEntity.getItemClass1(), iClassEntity.getItemClass2(), iClassEntity.getItemClass3(), iClassEntity.getItemClass4());
        iClassEntity.setItemClassMgntNo(itemClassInfo.getItemClassMgntNo());
        return iClassEntity;
    }

    private ItemEntity toItemEntity4OqLink(OqScmMenuModel item) {
        ItemEntity itemEntity = new ItemEntity();
        itemEntity.setItemId(item.getItemId());
        itemEntity.setOqMenuCd(item.getMenuCd());
        return itemEntity;
    }

    private ItemEntity toItemEntity(SaveItemInfolRequest saveItemInfolRequest) {
        ItemEntity entity = new ItemEntity();
        entity.setBrandId(saveItemInfolRequest.getBrandId());
        entity.setItemId(saveItemInfolRequest.getItemId());
        entity.setItemNm(saveItemInfolRequest.getItemNm());
        entity.setPchUnit(saveItemInfolRequest.getPchUnit());
        entity.setWeight(saveItemInfolRequest.getWeight());
        entity.setVolume(saveItemInfolRequest.getVolume());
        entity.setWidth(saveItemInfolRequest.getWidth());
        entity.setHeight(saveItemInfolRequest.getHeight());
        entity.setHigh(saveItemInfolRequest.getHigh());
        entity.setExpryYn(saveItemInfolRequest.getExpryYn());
        entity.setExpryPeriod(saveItemInfolRequest.getExpryPeriod());
        entity.setRtnDvsn(saveItemInfolRequest.getRtnDvsn());
        entity.setImpDvsn(saveItemInfolRequest.getImpDvsn());
        entity.setAssetInfo(saveItemInfolRequest.getAssetInfo());
        entity.setStckUnit(saveItemInfolRequest.getStckUnit());
        entity.setPodrPssblDvsn(saveItemInfolRequest.getPodrPssblDvsn());
        entity.setPchItemYn(saveItemInfolRequest.getPchItemYn());
        entity.setSaleItemYn(saveItemInfolRequest.getSaleItemYn());
        entity.setItemStCd(saveItemInfolRequest.getItemStCd());
        entity.setItemType(saveItemInfolRequest.getItemType());
        entity.setSaleUnit(saveItemInfolRequest.getSaleUnit());
        entity.setUnit(saveItemInfolRequest.getUnit());
        entity.setRtnPssblYn(saveItemInfolRequest.getRtnPssblYn());
        entity.setItemImageFileNo(saveItemInfolRequest.getItemImageFileNo());
        entity.setOneBoxQty(saveItemInfolRequest.getOneBoxQty());
        entity.setWeightUnit(saveItemInfolRequest.getWeightUnit());
        entity.setVolumeUnit(saveItemInfolRequest.getVolumeUnit());
        entity.setWidthUnit(saveItemInfolRequest.getWidthUnit());
        entity.setHeightUnit(saveItemInfolRequest.getHeightUnit());
        entity.setHighUnit(saveItemInfolRequest.getHighUnit());
        entity.setOqBrandId(saveItemInfolRequest.getOqBrandId());
        entity.setOqMenuCd(saveItemInfolRequest.getOqMenuCd());
        entity.setBarcode(saveItemInfolRequest.getBarcode());
        entity.setStoreDeliType(saveItemInfolRequest.getStoreDeliType());
        return entity;
    }

    @Transactional
    public void saveItemList(String userId, SaveItemListRequest saveItemListRequest) throws Exception {
        try {
            List<SaveItemListData> list = saveItemListRequest.getList();
            for (SaveItemListData item : list) {
                // 품목 아이디 부여  22  entity.setContentsNo(fnGetKeyNo(17));
                // entity.setItemId(fnGetDealNo(26, ""));  // 품목아이디 생성
                item.setItemId(sessionService.getKeyInfo("It").getKeyData()); // 품목아이디 생성
                ItemEntity entity = toItemEntity(userId, saveItemListRequest, item);
                itemMapper.insertItemInfo(userId, entity);
            }

            // 오더퀸 연동
            BrandEntity brandEntity = brandMapper.selectBrandInfo(saveItemListRequest.getBrandId());
            if (!StringUtil.isEmpty(brandEntity.getOqBrandId())) {
                List<OqScmMenuModel> menuList = new ArrayList<>();
                for (SaveItemListData item : list) {
                    OqScmMenuModel menu = new OqScmMenuModel();
                    menu.setItemId(item.getItemId()); // 품목아이디
                    menu.setItemNm(item.getItemNm()); // 품목명
                    menu.setBarcodeNo(item.getBarcode()); // 바코드번호
                    menuList.add(menu);
                }

                OqScmMenuRegReqModel req = new OqScmMenuRegReqModel();
                req.setBrandCd(brandEntity.getOqBrandId());
                req.setMenuList(menuList);
                OqScmMenuRegResModel res = oqlinkClient.regScmMenu(HmacUtil.makeHmacValue4OqLink(req), req);

                if (SystemMessageCode.BO_IF_OK.getId() == Integer.parseInt(res.getCode())) {
                    // 오더퀸 메뉴코드 업데이트
                    ItemEntity itemEntity = null;
                    for (OqScmMenuModel item : res.getMenuList()) {
                        itemEntity = toItemEntity4OqLink(item);
                        itemMapper.updateItemInfo4OqLinik(userId, itemEntity);
                    }
                } else {
                    throw new GlobalException(SystemMessageCode.valueOf(Integer.valueOf(res.getCode())));
                }
            }
        } catch (GlobalException e) {
            e.getStackTrace();
            throw new GlobalException(e.getSystemMessageCode());
        } catch (Exception e) {
            e.getStackTrace();
            throw new RuntimeException();
        }
    }

    private ItemEntity toItemEntity(String userId, SaveItemListRequest saveItemListRequest, SaveItemListData item) {
        ItemEntity entity = new ItemEntity();
        entity.setItemId(item.getItemId());
        entity.setItemNm(item.getItemNm());
        entity.setItemStCd("IS00"); // 사용
        entity.setBrandId(saveItemListRequest.getBrandId());
        entity.setUnit(DstrbtCostUnit.getCd(item.getUnitNm())); // 단위
        // entity.setPchUnit(item.getPchUnit()); // 구매단위
        // entity.setSaleUnit(item.getSaleUnit()); // 판매단위
        // entity.setStckUnit(item.getStckUnit()); // 재고단위
        entity.setWeight(item.getWeight());
        entity.setWeightUnit(WeightUnit.getCd(item.getWeightUnitNm()));
        entity.setVolume(item.getVolume());
        entity.setVolumeUnit(VolumeUnit.getCd(item.getVolumeUnitNm()));
        entity.setWidth(item.getWidth());
        entity.setWidthUnit(WidthUnit.getCd(item.getWidthUnitNm()));
        entity.setHeight(item.getHeight());
        entity.setHeightUnit(HeightUnit.getCd(item.getHeightUnitNm()));
        entity.setHigh(item.getHigh());
        entity.setHighUnit(HighUnit.getCd(item.getHighUnitNm()));
        entity.setPodrPssblDvsn(PodrPssblDvsn.getCd(item.getPodrPssblDvsnNm()));
        entity.setRtnPssblYn(item.getRtnPssblYn());
        entity.setExpryYn(item.getExpryYn());
        entity.setExpryPeriod(item.getExpryPeriod());
        entity.setRtnDvsn(RtnDvsn.getCd(item.getRtnDvsnNm()));
        entity.setImpDvsn(ImpDvsn.getCd(item.getImpDvsnNm()));
        entity.setPchItemYn(item.getPchItemYn());
        entity.setSaleItemYn(item.getSaleItemYn());
        entity.setItemType(ItemType.getCd(item.getItemTypeNm()));
        entity.setBarcode(item.getBarcode());
        entity.setStoreDeliTypeNm(item.getStoreDeliTypeNm());
        entity.setStoreDeliType(item.getStoreDeliTypeNm() == "본사배송" ? "10" : "20");
        entity.setSysRegId(userId);
        entity.setSysChgId(userId);
        return entity;
    }

    //
    /*
       품목분류 리스트 조회
     */
    public GridResponse selectItemClassMngGridList(String sBrandId, String sItemClass1, String sItemClass2, String sItemClass3, String sItemClass4, String sItemClassNm, GridRequest gridRequest) throws Exception {
        int gridPage = gridRequest.getGridPage();
        int gridSize = gridRequest.getGridSize();

        int gridRecords = itemMapper.selectItemClassMngGridCnt(sBrandId, sItemClass1, sItemClass2, sItemClass3, sItemClass4, sItemClassNm);
        int gridTotal = fnCalculateGridTotal(gridSize, gridRecords);
        List<ItemClassMngEntity> entities = itemMapper.selectItemClassMngGridList(sBrandId, sItemClass1, sItemClass2, sItemClass3, sItemClass4, sItemClassNm, gridRequest);
        List<Object> gridRows = entities.stream()
                .map(m -> ItemClassMngSearchResponse.builder()
                        .viewCd("R")
                        .brandId(m.getBrandId())
                        .itemClassMgntNo(m.getItemClassMgntNo())
                        .itemClassNm(m.getItemClassNm() == null ? "" : m.getItemClassNm())
                        .itemClass1(m.getItemClass1() == null ? "" : m.getItemClass1())
                        .itemClass2(m.getItemClass2() == null ? "" : m.getItemClass2())
                        .itemClass3(m.getItemClass3() == null ? "" : m.getItemClass3())
                        .itemClass4(m.getItemClass4() == null ? "" : m.getItemClass4())
                        .itemClass1Nm(m.getItemClass1Nm() == null ? "" : m.getItemLevel() >= 1 ? m.getItemClass1Nm() : "")
                        .itemClass2Nm(m.getItemClass2Nm() == null ? "" : m.getItemLevel() >= 2 ? m.getItemClass2Nm() : "")
                        .itemClass3Nm(m.getItemClass3Nm() == null ? "" : m.getItemLevel() >= 3 ? m.getItemClass3Nm() : "")
                        .itemClass4Nm(m.getItemClass4Nm() == null ? "" : m.getItemLevel() >= 4 ? m.getItemClass4Nm() : "")
                        .prnSeq(m.getPrnSeq() == null ? 99 : m.getPrnSeq())
                        .build())
                .collect(Collectors.toList());
        return GridResponse.toDTO(gridPage, gridTotal, gridRecords, gridRows);
    }

    /* 품목 분류 정보 */
    public ItemClassMngInfoResponse selectItemClassMngInfo(String brandId, String itemClassMgnNo) throws Exception {
        return ItemClassMngInfoResponse.toDTO(itemMapper.selectItemClassMngInfo(brandId, itemClassMgnNo));

    }

    /* 품목분류 저장 */
    @Transactional
    public void saveItemClassMngInfo(String userId, SaveItemClassMngInfolRequest saveItemClassMngInfolRequest) throws Exception {
        try {
            ItemClassMngEntity entity = new ItemClassMngEntity();
            entity.setBrandId(saveItemClassMngInfolRequest.getBrandId());
            entity.setItemClassNm(saveItemClassMngInfolRequest.getItemClassNm());
            entity.setItemClassMgntNo(saveItemClassMngInfolRequest.getItemClassMgntNo());
            entity.setItemClass1(saveItemClassMngInfolRequest.getItemClass1());
            entity.setItemClass2(saveItemClassMngInfolRequest.getItemClass2());
            entity.setItemClass3(saveItemClassMngInfolRequest.getItemClass3());
            entity.setItemClass4(saveItemClassMngInfolRequest.getItemClass4());
            entity.setPrnSeq(saveItemClassMngInfolRequest.getPrnSeq());

            if ("C".equals(saveItemClassMngInfolRequest.getViewCd())) {
                Integer itemLevel = 1;
                if (StringUtil.isEmpty(entity.getItemClass1())) {
                    // level 1 추가
                    itemLevel = 1;
                } else {
                    if (StringUtil.isEmpty(entity.getItemClass2())) {
                        // level 2 추가
                        itemLevel = 2;
                    } else {
                        if (StringUtil.isEmpty(entity.getItemClass3())) {
                            // level 3 추가
                            itemLevel = 3;
                        } else {
                            if (StringUtil.isEmpty(entity.getItemClass4())) {
                                // level 4 추가
                                itemLevel = 4;
                            } else {
                                // 오류
                                throw new GlobalException(SystemMessageCode.ERR_ITEMCLASS);
                            }
                        }
                    }
                }

                System.out.println("=classLevel============================>" + itemLevel);
                entity.setItemLevel(itemLevel);
                int classNum = itemMapper.selectItemClassMaxClass(entity.getBrandId(), entity.getItemLevel(), entity.getItemClass1(), entity.getItemClass2(), entity.getItemClass3(), entity.getItemClass4());
                if (itemLevel == 1) {
                    entity.setItemClass1(Integer.toString(classNum));
                } else if (itemLevel == 2) {
                    entity.setItemClass2(Integer.toString(classNum));
                } else if (itemLevel == 3) {
                    entity.setItemClass3(Integer.toString(classNum));
                } else if (itemLevel == 4) {
                    entity.setItemClass4(Integer.toString(classNum));
                }
                entity.setItemClassMgntNo(fnGetKeyNo(31));        // 품목분류 아이디 부여

                itemMapper.insertItemClassMngInfo(userId, entity);
            } else if ("U".equals(saveItemClassMngInfolRequest.getViewCd())) {
                itemMapper.updateItemClassMngInfo(userId, entity);
            }

        } catch (GlobalException e) {
            e.getStackTrace();
            throw new GlobalException(e.getSystemMessageCode());
        } catch (Exception e) {
            e.getStackTrace();
            throw new RuntimeException();
        }
    }

    /* 품목분류 삭제 */
    @Transactional
    public void delItemClassMngInfo(String userId, String brandId, String itemClassMgntNo) throws Exception {
        try {
            List<ItemClassMngEntity> icmngList = new ArrayList<ItemClassMngEntity>();

            String[] icmNoList = itemClassMgntNo.split(",");
            for (int i = 0; i < icmNoList.length; i++) {
                List<ItemClassMngEntity> belongList = itemMapper.selectItemClassBeLongList(brandId, icmNoList[i]);
                for (int j = 0; j < belongList.size(); j++) {
                    ItemClassMngEntity delEntity = belongList.get(j);
                    delEntity.setItemClassMgntNo(delEntity.getItemClassMgntNo());
                    icmngList.add(delEntity);
                }
            }
            itemMapper.deleteItemClassMngInfo(userId, icmngList);
        } catch (GlobalException e) {
            e.getStackTrace();
            throw new GlobalException(e.getSystemMessageCode());
        } catch (Exception e) {
            e.getStackTrace();
            throw new RuntimeException();
        }
    }

    /* 품목 분류 level 리스트  */

    public ItemClassLevelResponse searchItemClassList(String sBrandId, int sItemLevel, String sItemClass1, String sItemClass2, String sItemClass3) throws Exception {
        List<ItemClassMngEntity> entities = itemMapper.selectItemLevelList(sBrandId, sItemLevel, sItemClass1, sItemClass2, sItemClass3);
        return ItemClassLevelResponse.toDTO(entities);
    }


    /* 바코드 품목 정보 조회 */
    public BarcodeItemInfoResponse selectBarcodeItemMInfo(String sBrandId, String sStoreId, String sBarcode) throws Exception {
        ItemUnitEntity entity = null;

        if (StringUtil.isEmpty(sStoreId)) {
            entity = itemUnitMapper.selectBarcodeItemBrandUnitInfo(sBrandId, sBarcode);
        } else {
            entity = itemUnitMapper.selectBarcodeItemStoreUnitInfo(sBrandId, sStoreId, sBarcode);
        }
        return BarcodeItemInfoResponse.toDTO(entity);
    }


    public StroeWishListResponse selectPopWishGridList(String sStoreId) throws Exception {
        List<ItemUnitEntity> entities = itemUnitMapper.selectPopWishGridList(sStoreId);
        return StroeWishListResponse.toDTO(entities);
    }

    /*
     팝업 위시리스트 품목 리스트 조회
    */
    public GridResponse selectPopItemGridWishList(String sBrandId, String sStoreId, String sWishListMgntNo, GridRequest gridRequest) throws Exception {
        int gridPage = gridRequest.getGridPage();
        int gridSize = gridRequest.getGridSize();

        int gridRecords = 0;
        int gridTotal = 0;

        gridRecords = itemUnitMapper.selectPopItemStoreUnitWishGridCnt(sBrandId, sStoreId, sWishListMgntNo);
        gridTotal = fnCalculateGridTotal(gridSize, gridRecords);
        List<ItemUnitEntity> entities = itemUnitMapper.selectPopItemStoreUnitWishGridList(sBrandId, sStoreId, sWishListMgntNo, gridRequest);

        List<Object> gridRows = entities.stream()
                .map(m -> ItemPopSearchResponse.builder()
                        .viewCd("R")
                        .itemId(m.getItemId())
                        .itemNm(m.getItemNm())
                        .podrPssblDvsn(m.getPodrPssblDvsn())
                        .podrPssblDvsnNm(m.getPodrPssblDvsnNm())
                        .unit(m.getUnit())
                        .unitNm(m.getUnitNm())
                        .unitAmt(m.getUnitAmt())
                        .stddQty(m.getStddQty())
                        .unitGubun(m.getUnitGubun())
                        .spplyId(m.getSpplyId())
                        .spplyNm(m.getSpplyNm() == null ? "" : m.getSpplyNm())
                        .brandUnitUnqNo(m.getBrandUnitUnqNo())
                        .storeUnitUnqNo(m.getStoreUnitUnqNo() == null ? "" : m.getStoreUnitUnqNo())
                        .whsPassQty(m.getWhsPassQty() == null ? 0 : m.getWhsPassQty())
                        .build())
                .collect(Collectors.toList());
        return GridResponse.toDTO(gridPage, gridTotal, gridRecords, gridRows);
    }


    /*
      팝업 품목 리스트 조회
     */
    public GridResponse selectPopLackStockItemGridList(String sBrandId, String sStoreId, String sWhsId, GridRequest gridRequest) throws Exception {
        int gridPage = gridRequest.getGridPage();
        int gridSize = gridRequest.getGridSize();

        int gridRecords = 0;
        int gridTotal = 0;
        List<ItemUnitEntity> entities = new ArrayList<ItemUnitEntity>();

        int podrNum = ApiConstants.LACK_STOCK_PODR_NUM;

        if (StringUtil.isEmpty(sStoreId)) {
            gridRecords = itemUnitMapper.selectPopBrandItemLackStockGridCnt(sBrandId, sWhsId);
            gridTotal = fnCalculateGridTotal(gridSize, gridRecords);
            entities = itemUnitMapper.selectPopBrandItemLackStockGridList(sBrandId, sWhsId, podrNum, gridRequest);
        } else {
            gridRecords = itemUnitMapper.selectPopStoreItemLackStockGridCnt(sBrandId, sStoreId, sWhsId);
            gridTotal = fnCalculateGridTotal(gridSize, gridRecords);
            entities = itemUnitMapper.selectPopStoreItemLackStockGridList(sBrandId, sStoreId, sWhsId, podrNum, gridRequest);
        }

        List<Object> gridRows = entities.stream()
                .map(m -> ItemPopLackStockSearchResponse.builder()
                        .viewCd("R")
                        .whsNm(m.getWhsNm() == null ? "" : m.getWhsNm())
                        .itemId(m.getItemId())
                        .itemNm(m.getItemNm())
                        .unit(m.getUnit())
                        .unitNm(m.getUnitNm())
                        .podrPssblDvsn(m.getPodrPssblDvsn())
                        .podrPssblDvsnNm(m.getPodrPssblDvsnNm())
                        .unitAmt(m.getUnitAmt())
                        .brandUnitUnqNo(m.getBrandUnitUnqNo())
                        .storeUnitUnqNo(m.getStoreUnitUnqNo() == null ? "" : m.getStoreUnitUnqNo())
                        .stckQty(m.getStckQty() == null ? 0 : m.getStckQty())
                        .pchReqQty(m.getPodrQty() == null ? 0 : m.getPodrQty())
                        .curQty(m.getCurQty() == null ? 0 : m.getCurQty())
                        .prpStckQty(m.getPrpStckQty() == null ? 0 : m.getPrpStckQty())
                        .unitGubun(m.getUnitGubun())
                        .build())
                .collect(Collectors.toList());
        return GridResponse.toDTO(gridPage, gridTotal, gridRecords, gridRows);
    }

    public GridResponse selectPopWishListItem(String userId, String wishListMgntNo, String brandId, String storeId, GridRequest gridRequest) throws Exception {

        int gridPage = gridRequest.getGridPage();
        int gridSize = gridRequest.getGridSize();

        int gridRecords = itemUnitMapper.selectPopWishListItemCnt(userId, wishListMgntNo, brandId, storeId);
        int gridTotal = fnCalculateGridTotal(gridSize, gridRecords);

        List<ItemUnitEntity> entities = itemUnitMapper.selectPopWishListItem(userId, wishListMgntNo, brandId, storeId, gridRequest);
        List<Object> gridRows = entities.stream().
                map(m -> ItemPopSearchResponse.builder()
                        .viewCd("R")
                        .itemId(m.getItemId())
                        .itemNm(m.getItemNm())
                        .podrPssblDvsn(m.getPodrPssblDvsn())
                        .podrPssblDvsnNm(m.getPodrPssblDvsnNm())
                        .unit(m.getUnit())
                        .unitNm(m.getUnitNm())
                        .unitAmt(m.getUnitAmt())
                        .stddQty(m.getStddQty())
                        .unitGubun(m.getUnitGubun())
                        .spplyId(m.getSpplyId())
                        .spplyNm(m.getSpplyNm())
                        .brandUnitUnqNo(m.getBrandUnitUnqNo())
                        .storeUnitUnqNo(m.getStoreUnitUnqNo() == null ? "" : m.getStoreUnitUnqNo())
                        .build())
                .collect(Collectors.toList()
                );
        return GridResponse.toDTO(gridPage, gridTotal, gridRecords, gridRows);
    }

    public StroeWhsListResponse selectPopWhsGridList(String sBrandId, String sStoreId) throws Exception {
        List<WhsMgntBaseInfoEntity> entities = whsMngMapper.selectWhsList(sBrandId, sStoreId);
        return StroeWhsListResponse.toDTO(entities);
    }
}
